"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inspectaccessresponse import InspectAccessResponse, InspectAccessResponseTypedDict
from factify.types import BaseModel
from factify.utils import FieldMetadata, PathParamMetadata
from typing import Dict, List
from typing_extensions import Annotated, TypedDict


class InspectDocumentAccessRequestTypedDict(TypedDict):
    document_id: str
    r"""Document ID."""


class InspectDocumentAccessRequest(BaseModel):
    document_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Document ID."""


class InspectDocumentAccessResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: InspectAccessResponseTypedDict


class InspectDocumentAccessResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: InspectAccessResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from factify import models, utils
from factify._hooks import SDKHooks
from factify.types import OptionalNullable, UNSET
import httpx
import importlib
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from factify.accessrequests import AccessRequests
    from factify.api_keys import APIKeys
    from factify.documents import Documents
    from factify.entrypages import EntryPages
    from factify.organizations import Organizations
    from factify.policies import Policies
    from factify.sharing import Sharing
    from factify.timelines import Timelines
    from factify.usage import Usage
    from factify.userpreferences import UserPreferences
    from factify.users import Users
    from factify.versions import Versions


class Factify(BaseSDK):
    r"""Factify API: Factify API enables you to create, manage, and control access to legally-binding documents that replace PDFs.

    ## Authentication

    Bearer authentication header of the form Bearer <token>, where <token> is your auth token.

    ```
    Authorization: Bearer ffy...
    ```

    ## Rate Limiting

    - 1000 requests per minute per API key
    - 100 requests per minute for resource creation (POST)

    ## Errors

    Factify uses conventional HTTP status codes and returns structured error responses
    using the [ConnectRPC error format](https://connectrpc.com/docs/protocol#error-codes):

    ```json
    {
    \"code\": \"invalid_argument\",
    \"message\": \"limit must be positive\",
    \"details\": [
    {
    \"reason\": \"invalid_field_value\",
    \"request_id\": \"req_01arwx4k8xrgqskvxq69gdn019\",
    \"param\": \"limit\"
    }
    ]
    }
    ```

    """

    api_keys: "APIKeys"
    r"""Generate and manage API keys for authentication."""
    documents: "Documents"
    r"""Create and manage legally-binding documents."""
    access_requests: "AccessRequests"
    entry_pages: "EntryPages"
    sharing: "Sharing"
    policies: "Policies"
    r"""Attach and manage access policies for documents."""
    timelines: "Timelines"
    versions: "Versions"
    r"""Document version tracking and management."""
    users: "Users"
    r"""User profile and account information."""
    organizations: "Organizations"
    r"""Create and manage organizations."""
    usage: "Usage"
    r"""Query API usage and quota information for your organization."""
    user_preferences: "UserPreferences"
    _sub_sdk_map = {
        "api_keys": ("factify.api_keys", "APIKeys"),
        "documents": ("factify.documents", "Documents"),
        "access_requests": ("factify.accessrequests", "AccessRequests"),
        "entry_pages": ("factify.entrypages", "EntryPages"),
        "sharing": ("factify.sharing", "Sharing"),
        "policies": ("factify.policies", "Policies"),
        "timelines": ("factify.timelines", "Timelines"),
        "versions": ("factify.versions", "Versions"),
        "users": ("factify.users", "Users"),
        "organizations": ("factify.organizations", "Organizations"),
        "usage": ("factify.usage", "Usage"),
        "user_preferences": ("factify.userpreferences", "UserPreferences"),
    }

    def __init__(
        self,
        bearer_auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        url_params: Optional[Dict[str, str]] = None,
        server_url: Optional[str] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param bearer_auth: The bearer_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(bearer_auth):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(bearer_auth=bearer_auth())
        else:
            security = models.Security(bearer_auth=bearer_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationinvitestatus import OrganizationInviteStatus
from .user import User, UserTypedDict
from datetime import datetime
from factify.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OrganizationInviteTypedDict(TypedDict):
    r"""OrganizationInvite represents an invitation to join an organization.
    accepted_fields_consistency // accepted_at and accepted_by must both be set or both be unset

    """

    created_at: datetime
    r"""When the invitation was created."""
    email: str
    r"""Email address of the recipient."""
    expires_at: datetime
    r"""When the invitation expires."""
    id: str
    r"""Unique identifier for the invitation."""
    organization_id: str
    r"""Organization the invitation is for."""
    sender: UserTypedDict
    r"""User represents a human user account.
    Returned by GetUser (endpoint deferred).
    """
    status: OrganizationInviteStatus
    accepted_at: NotRequired[datetime]
    r"""When the invitation was accepted (if accepted)."""
    accepted_by: NotRequired[UserTypedDict]
    r"""User represents a human user account.
    Returned by GetUser (endpoint deferred).
    """
    message: NotRequired[Nullable[str]]
    r"""Custom message from sender to recipient (if provided).
    Max 2000 bytes to support ~500 multibyte characters.
    """


class OrganizationInvite(BaseModel):
    r"""OrganizationInvite represents an invitation to join an organization.
    accepted_fields_consistency // accepted_at and accepted_by must both be set or both be unset

    """

    created_at: datetime
    r"""When the invitation was created."""

    email: str
    r"""Email address of the recipient."""

    expires_at: datetime
    r"""When the invitation expires."""

    id: str
    r"""Unique identifier for the invitation."""

    organization_id: str
    r"""Organization the invitation is for."""

    sender: User
    r"""User represents a human user account.
    Returned by GetUser (endpoint deferred).
    """

    status: OrganizationInviteStatus

    accepted_at: Optional[datetime] = None
    r"""When the invitation was accepted (if accepted)."""

    accepted_by: Optional[User] = None
    r"""User represents a human user account.
    Returned by GetUser (endpoint deferred).
    """

    message: OptionalNullable[str] = UNSET
    r"""Custom message from sender to recipient (if provided).
    Max 2000 bytes to support ~500 multibyte characters.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["accepted_at", "accepted_by", "message"])
        nullable_fields = set(["message"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accesslevel import AccessLevel
from .documentpermissionset import DocumentPermissionSet, DocumentPermissionSetTypedDict
from .processingstatus import ProcessingStatus
from .subject import Subject, SubjectTypedDict
from .versionref import VersionRef, VersionRefTypedDict
from datetime import datetime
from factify.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DocumentTypedDict(TypedDict):
    r"""Document represents a Factify document."""

    access_level: AccessLevel
    created_at: datetime
    r"""Timestamp when document was created."""
    created_by: SubjectTypedDict
    r"""Subject is a lightweight reference to either a user or a bot.
    Used in embedded fields like created_by.
    For full details, call GetUser or GetBot with the subject's id.
    subject_type_matches_id_prefix // id prefix must match type (user_ for user, bot_ for bot)

    """
    id: str
    r"""Unique ID for the document.
    Pattern: doc_[0-9a-hjkmnp-tv-z]{26}
    """
    processing_status: ProcessingStatus
    title: str
    r"""Document title."""
    url: str
    r"""URL for accessing the document on Factify."""
    current_version: NotRequired[Nullable[VersionRefTypedDict]]
    r"""Reference to the current version, if any."""
    description: NotRequired[Nullable[str]]
    r"""Optional document description."""
    general_access: NotRequired[AccessLevel]
    is_demo: NotRequired[bool]
    r"""Whether this document is a demo document created during FTUE onboarding."""
    last_viewed_at: NotRequired[datetime]
    r"""Timestamp when the authenticated user last viewed this document.
    Absent if the user has never viewed it.
    """
    owner_organization_id: NotRequired[str]
    r"""Organization that owns this document (via the document creator).
    Empty if the owner's organization could not be resolved.
    """
    permission_set: NotRequired[DocumentPermissionSetTypedDict]
    r"""DocumentPermissionSet contains the permissions the authenticated user has on a document."""
    shared_at: NotRequired[datetime]
    r"""Timestamp when this document was last shared to the authenticated user.
    Absent if the document was never shared to them.
    """
    thumbnail_url: NotRequired[str]
    r"""URL of the document thumbnail image."""
    trashed_at: NotRequired[datetime]
    r"""Timestamp when the document was trashed, if applicable.
    Absent if the document is not trashed.
    """
    updated_at: NotRequired[datetime]
    r"""Timestamp when document was last updated."""


class Document(BaseModel):
    r"""Document represents a Factify document."""

    access_level: AccessLevel

    created_at: datetime
    r"""Timestamp when document was created."""

    created_by: Subject
    r"""Subject is a lightweight reference to either a user or a bot.
    Used in embedded fields like created_by.
    For full details, call GetUser or GetBot with the subject's id.
    subject_type_matches_id_prefix // id prefix must match type (user_ for user, bot_ for bot)

    """

    id: str
    r"""Unique ID for the document.
    Pattern: doc_[0-9a-hjkmnp-tv-z]{26}
    """

    processing_status: ProcessingStatus

    title: str
    r"""Document title."""

    url: str
    r"""URL for accessing the document on Factify."""

    current_version: OptionalNullable[VersionRef] = UNSET
    r"""Reference to the current version, if any."""

    description: OptionalNullable[str] = UNSET
    r"""Optional document description."""

    general_access: Optional[AccessLevel] = None

    is_demo: Optional[bool] = None
    r"""Whether this document is a demo document created during FTUE onboarding."""

    last_viewed_at: Optional[datetime] = None
    r"""Timestamp when the authenticated user last viewed this document.
    Absent if the user has never viewed it.
    """

    owner_organization_id: Optional[str] = None
    r"""Organization that owns this document (via the document creator).
    Empty if the owner's organization could not be resolved.
    """

    permission_set: Optional[DocumentPermissionSet] = None
    r"""DocumentPermissionSet contains the permissions the authenticated user has on a document."""

    shared_at: Optional[datetime] = None
    r"""Timestamp when this document was last shared to the authenticated user.
    Absent if the document was never shared to them.
    """

    thumbnail_url: Optional[str] = None
    r"""URL of the document thumbnail image."""

    trashed_at: Optional[datetime] = None
    r"""Timestamp when the document was trashed, if applicable.
    Absent if the document is not trashed.
    """

    updated_at: Optional[datetime] = None
    r"""Timestamp when document was last updated."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "current_version",
                "description",
                "general_access",
                "is_demo",
                "last_viewed_at",
                "owner_organization_id",
                "permission_set",
                "shared_at",
                "thumbnail_url",
                "trashed_at",
                "updated_at",
            ]
        )
        nullable_fields = set(["current_version", "description"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .processingstatus import ProcessingStatus
from .subject import Subject, SubjectTypedDict
from datetime import datetime
from factify.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class VersionTypedDict(TypedDict):
    r"""Version represents a document version."""

    created_at: datetime
    r"""Timestamp when version was created."""
    created_by: SubjectTypedDict
    r"""Subject is a lightweight reference to either a user or a bot.
    Used in embedded fields like created_by.
    For full details, call GetUser or GetBot with the subject's id.
    subject_type_matches_id_prefix // id prefix must match type (user_ for user, bot_ for bot)

    """
    document_id: str
    r"""Parent document ID.
    Pattern: doc_[0-9a-hjkmnp-tv-z]{26}
    """
    id: str
    r"""Unique ID for the version.
    Pattern: ver_[0-9a-hjkmnp-tv-z]{26}
    """
    processing_status: ProcessingStatus
    url: str
    r"""URL for accessing this version on Factify."""
    description: NotRequired[Nullable[str]]
    r"""Description of changes in this version."""
    title: NotRequired[Nullable[str]]
    r"""Optional version title."""


class Version(BaseModel):
    r"""Version represents a document version."""

    created_at: datetime
    r"""Timestamp when version was created."""

    created_by: Subject
    r"""Subject is a lightweight reference to either a user or a bot.
    Used in embedded fields like created_by.
    For full details, call GetUser or GetBot with the subject's id.
    subject_type_matches_id_prefix // id prefix must match type (user_ for user, bot_ for bot)

    """

    document_id: str
    r"""Parent document ID.
    Pattern: doc_[0-9a-hjkmnp-tv-z]{26}
    """

    id: str
    r"""Unique ID for the version.
    Pattern: ver_[0-9a-hjkmnp-tv-z]{26}
    """

    processing_status: ProcessingStatus

    url: str
    r"""URL for accessing this version on Factify."""

    description: OptionalNullable[str] = UNSET
    r"""Description of changes in this version."""

    title: OptionalNullable[str] = UNSET
    r"""Optional version title."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "title"])
        nullable_fields = set(["description", "title"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


ProcessingOutcome = Literal[
    "PROCESSING_OUTCOME_UNSPECIFIED",
    "PROCESSING_OUTCOME_STARTED",
    "PROCESSING_OUTCOME_ALREADY_PROCESSING",
    "PROCESSING_OUTCOME_ALREADY_COMPLETED",
    "PROCESSING_OUTCOME_FAILED",
]
r"""ProcessingOutcome describes the result of a processing trigger request."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .version import Version, VersionTypedDict
from factify.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ListVersionsResponseTypedDict(TypedDict):
    r"""ListVersionsResponse contains a page of versions."""

    pagination: PaginationTypedDict
    r"""Pagination contains cursor-based pagination metadata.
    Follows Google AIP-158 for pagination field naming.
    """
    items: NotRequired[List[VersionTypedDict]]
    r"""List of versions."""


class ListVersionsResponse(BaseModel):
    r"""ListVersionsResponse contains a page of versions."""

    pagination: Pagination
    r"""Pagination contains cursor-based pagination metadata.
    Follows Google AIP-158 for pagination field naming.
    """

    items: Optional[List[Version]] = None
    r"""List of versions."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["items"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .documentrole import DocumentRole
from .updatedocumentaccessresponse import (
    UpdateDocumentAccessResponse,
    UpdateDocumentAccessResponseTypedDict,
)
from factify.types import BaseModel
from factify.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Dict, List
from typing_extensions import Annotated, TypedDict


class UpdateDocumentAccessUpdateDocumentAccessRequestTypedDict(TypedDict):
    r"""UpdateDocumentAccessRequest updates a single recipient's role."""

    document_role: DocumentRole


class UpdateDocumentAccessUpdateDocumentAccessRequest(BaseModel):
    r"""UpdateDocumentAccessRequest updates a single recipient's role."""

    document_role: DocumentRole


class UpdateDocumentAccessRequestTypedDict(TypedDict):
    document_id: str
    r"""Document ID."""
    recipient_id: str
    r"""Recipient ID (TypeID: user_xxx or bot_xxx)."""
    body: UpdateDocumentAccessUpdateDocumentAccessRequestTypedDict


class UpdateDocumentAccessRequest(BaseModel):
    document_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Document ID."""

    recipient_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Recipient ID (TypeID: user_xxx or bot_xxx)."""

    body: Annotated[
        UpdateDocumentAccessUpdateDocumentAccessRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class UpdateDocumentAccessResponseResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: UpdateDocumentAccessResponseTypedDict


class UpdateDocumentAccessResponseResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: UpdateDocumentAccessResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from factify.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CreateAPIKeyRequestTypedDict(TypedDict):
    r"""CreateApiKeyRequest contains the data needed to create a new API key."""

    name: str
    r"""Human-readable name for the key (e.g., \"Production\", \"Staging\")."""
    organization_id: str
    r"""Organization ID to create the key for.
    Pattern: org_[0-9a-hjkmnp-tv-z]{26}
    """
    expires_at: NotRequired[datetime]
    r"""Expiration timestamp. If not set, the key does not expire.
    When set, must be in the future and within 1 year.
    Security note: Keys without expiration should be rotated periodically.
    timestamp.gt_now = true
    timestamp.within = 8760h0m0s

    """


class CreateAPIKeyRequest(BaseModel):
    r"""CreateApiKeyRequest contains the data needed to create a new API key."""

    name: str
    r"""Human-readable name for the key (e.g., \"Production\", \"Staging\")."""

    organization_id: str
    r"""Organization ID to create the key for.
    Pattern: org_[0-9a-hjkmnp-tv-z]{26}
    """

    expires_at: Optional[datetime] = None
    r"""Expiration timestamp. If not set, the key does not expire.
    When set, must be in the future and within 1 year.
    Security note: Keys without expiration should be rotated periodically.
    timestamp.gt_now = true
    timestamp.within = 8760h0m0s

    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["expires_at"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from factify.types import BaseModel
from typing_extensions import TypedDict


class PolicyTypedDict(TypedDict):
    r"""Policy represents a governance policy."""

    created_at: datetime
    r"""Timestamp when policy was created."""
    id: str
    r"""Unique ID for the policy.
    Pattern: pol_[0-9a-hjkmnp-tv-z]{26}
    """
    name: str
    r"""Policy name."""


class Policy(BaseModel):
    r"""Policy represents a governance policy."""

    created_at: datetime
    r"""Timestamp when policy was created."""

    id: str
    r"""Unique ID for the policy.
    Pattern: pol_[0-9a-hjkmnp-tv-z]{26}
    """

    name: str
    r"""Policy name."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resendorganizationinviteresponse import (
    ResendOrganizationInviteResponse,
    ResendOrganizationInviteResponseTypedDict,
)
from factify.types import BaseModel
from factify.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Dict, List
from typing_extensions import Annotated, TypedDict


class ResendOrganizationInviteResendOrganizationInviteRequestTypedDict(TypedDict):
    r"""ResendOrganizationInviteRequest identifies an invitation to resend."""


class ResendOrganizationInviteResendOrganizationInviteRequest(BaseModel):
    r"""ResendOrganizationInviteRequest identifies an invitation to resend."""


class ResendOrganizationInviteRequestTypedDict(TypedDict):
    organization_id: str
    r"""Organization the invitation belongs to.
    Pattern: org_[0-9a-hjkmnp-tv-z]{26}
    """
    invite_id: str
    r"""Invitation ID to resend.
    Pattern: inv_[0-9a-hjkmnp-tv-z]{26}
    """
    body: ResendOrganizationInviteResendOrganizationInviteRequestTypedDict


class ResendOrganizationInviteRequest(BaseModel):
    organization_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Organization the invitation belongs to.
    Pattern: org_[0-9a-hjkmnp-tv-z]{26}
    """

    invite_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Invitation ID to resend.
    Pattern: inv_[0-9a-hjkmnp-tv-z]{26}
    """

    body: Annotated[
        ResendOrganizationInviteResendOrganizationInviteRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class ResendOrganizationInviteResponseResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ResendOrganizationInviteResponseTypedDict


class ResendOrganizationInviteResponseResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ResendOrganizationInviteResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from factify import errors, models, utils
from factify._hooks import HookContext
from factify.types import OptionalNullable, UNSET
from factify.utils.unmarshal_json_response import unmarshal_json_response
from jsonpath import JSONPath
from typing import Any, Awaitable, Dict, List, Mapping, Optional, Union


class Documents(BaseSDK):
    r"""Create and manage legally-binding documents."""

    def list(
        self,
        *,
        page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        created_by_id: Optional[List[str]] = None,
        access_level: Optional[List[models.AccessLevel]] = None,
        processing_status: Optional[List[models.ProcessingStatus]] = None,
        sort: Optional[str] = None,
        query: Optional[str] = None,
        ownership: Optional[List[models.DocumentOwnership]] = None,
        trash_state: Optional[List[models.DocumentTrashState]] = None,
        organization_scope: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ListDocumentsResponseResponse]:
        r"""List documents

        List documents with cursor-based pagination.

        :param page_token: Opaque pagination token from a previous response.
            Pass next_page_token or prev_page_token from a previous response to continue pagination.
            Empty or omitted for the first page.
        :param page_size: Maximum number of items to return per page (1-100). Default: 50.
        :param created_after: Return results after this timestamp (inclusive).
        :param created_before: Return results before this timestamp (inclusive).
        :param created_by_id: Filter by creator ID(s) (user or bot). Returns documents matching ANY of the specified IDs.
            REST: ?created_by_id=user_01h2xcejqtf2nbrexx3vqjhp41 or ?created_by_id=user_xxx&created_by_id=bot_yyy
        :param access_level: Filter by access level(s). Returns documents matching ANY of the specified levels.
            REST: ?access_level=private or ?access_level=private&access_level=organization
        :param processing_status: Filter by processing status(es). Returns documents matching ANY of the specified statuses.
            REST: ?processing_status=ready or ?processing_status=processing&processing_status=ready
        :param sort: Sort field and direction. Prefix with `-` for descending order.
            Allowed values: created_at, updated_at, name, last_viewed_at, last_shared_at.
            Default (omitted): created_at descending.
            REST: ?sort=last_viewed_at or ?sort=-name
        :param query: Full-text search filter. Case-insensitive substring match on document name and description.
            REST: ?query=budget
        :param ownership: Ownership filter. Returns documents matching the specified ownership state.
            REST: ?ownership=owned or ?ownership=not_owned
        :param trash_state: Trash state filter. Returns documents matching the specified trash state.
            REST: ?trash_state=active or ?trash_state=trashed or ?trash_state=active&trash_state=trashed
            Default (omitted): active documents only.
        :param organization_scope: Organization scope filter. When true, restrict to documents within the user's organization.
            REST: ?organization_scope=true
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListDocumentsRequest(
            page_token=page_token,
            page_size=page_size,
            created_after=created_after,
            created_before=created_before,
            created_by_id=created_by_id,
            access_level=access_level,
            processing_status=processing_status,
            sort=sort,
            query=query,
            ownership=ownership,
            trash_state=trash_state,
            organization_scope=organization_scope,
        )

        req = self._build_request(
            method="GET",
            path="/v1beta/documents",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listDocuments",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.ListDocumentsResponseResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])

            next_cursor = JSONPath("$.pagination.next_page_token").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return None

            return self.list(
                page_token=next_cursor,
                page_size=page_size,
                created_after=created_after,
                created_before=created_before,
                created_by_id=created_by_id,
                access_level=access_level,
                processing_status=processing_status,
                sort=sort,
                query=query,
                ownership=ownership,
                trash_state=trash_state,
                organization_scope=organization_scope,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListDocumentsResponseResponse(
                result=unmarshal_json_response(models.ListDocumentsResponse, http_res),
                next=next_func,
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        created_after: Optional[datetime] = None,
        created_before: Optional[datetime] = None,
        created_by_id: Optional[List[str]] = None,
        access_level: Optional[List[models.AccessLevel]] = None,
        processing_status: Optional[List[models.ProcessingStatus]] = None,
        sort: Optional[str] = None,
        query: Optional[str] = None,
        ownership: Optional[List[models.DocumentOwnership]] = None,
        trash_state: Optional[List[models.DocumentTrashState]] = None,
        organization_scope: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ListDocumentsResponseResponse]:
        r"""List documents

        List documents with cursor-based pagination.

        :param page_token: Opaque pagination token from a previous response.
            Pass next_page_token or prev_page_token from a previous response to continue pagination.
            Empty or omitted for the first page.
        :param page_size: Maximum number of items to return per page (1-100). Default: 50.
        :param created_after: Return results after this timestamp (inclusive).
        :param created_before: Return results before this timestamp (inclusive).
        :param created_by_id: Filter by creator ID(s) (user or bot). Returns documents matching ANY of the specified IDs.
            REST: ?created_by_id=user_01h2xcejqtf2nbrexx3vqjhp41 or ?created_by_id=user_xxx&created_by_id=bot_yyy
        :param access_level: Filter by access level(s). Returns documents matching ANY of the specified levels.
            REST: ?access_level=private or ?access_level=private&access_level=organization
        :param processing_status: Filter by processing status(es). Returns documents matching ANY of the specified statuses.
            REST: ?processing_status=ready or ?processing_status=processing&processing_status=ready
        :param sort: Sort field and direction. Prefix with `-` for descending order.
            Allowed values: created_at, updated_at, name, last_viewed_at, last_shared_at.
            Default (omitted): created_at descending.
            REST: ?sort=last_viewed_at or ?sort=-name
        :param query: Full-text search filter. Case-insensitive substring match on document name and description.
            REST: ?query=budget
        :param ownership: Ownership filter. Returns documents matching the specified ownership state.
            REST: ?ownership=owned or ?ownership=not_owned
        :param trash_state: Trash state filter. Returns documents matching the specified trash state.
            REST: ?trash_state=active or ?trash_state=trashed or ?trash_state=active&trash_state=trashed
            Default (omitted): active documents only.
        :param organization_scope: Organization scope filter. When true, restrict to documents within the user's organization.
            REST: ?organization_scope=true
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListDocumentsRequest(
            page_token=page_token,
            page_size=page_size,
            created_after=created_after,
            created_before=created_before,
            created_by_id=created_by_id,
            access_level=access_level,
            processing_status=processing_status,
            sort=sort,
            query=query,
            ownership=ownership,
            trash_state=trash_state,
            organization_scope=organization_scope,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1beta/documents",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listDocuments",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Awaitable[Optional[models.ListDocumentsResponseResponse]]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])

            async def empty_result():
                return None

            next_cursor = JSONPath("$.pagination.next_page_token").parse(body)

            if len(next_cursor) == 0:
                return empty_result()

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return empty_result()

            return self.list_async(
                page_token=next_cursor,
                page_size=page_size,
                created_after=created_after,
                created_before=created_before,
                created_by_id=created_by_id,
                access_level=access_level,
                processing_status=processing_status,
                sort=sort,
                query=query,
                ownership=ownership,
                trash_state=trash_state,
                organization_scope=organization_scope,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListDocumentsResponseResponse(
                result=unmarshal_json_response(models.ListDocumentsResponse, http_res),
                next=next_func,
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        payload: Union[
            models.CreateDocumentRequestPayload,
            models.CreateDocumentRequestPayloadTypedDict,
        ],
        title: str,
        access_level: OptionalNullable[models.AccessLevel] = UNSET,
        description: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateDocumentResponseResponse:
        r"""Create a document

        Creates a new document by uploading a PDF file.

        :param payload: PDF file to upload
        :param title: Document title.
        :param access_level: Document access level. Defaults to organization if user belongs to an org, otherwise private.
        :param description: Optional document description.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateDocumentRequest(
            access_level=access_level,
            description=description,
            payload=utils.get_pydantic_model(
                payload, models.CreateDocumentRequestPayload
            ),
            title=title,
        )

        req = self._build_request(
            method="POST",
            path="/v1beta/documents",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "multipart", models.CreateDocumentRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CreateDocumentResponseResponse(
                result=unmarshal_json_response(models.CreateDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        payload: Union[
            models.CreateDocumentRequestPayload,
            models.CreateDocumentRequestPayloadTypedDict,
        ],
        title: str,
        access_level: OptionalNullable[models.AccessLevel] = UNSET,
        description: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CreateDocumentResponseResponse:
        r"""Create a document

        Creates a new document by uploading a PDF file.

        :param payload: PDF file to upload
        :param title: Document title.
        :param access_level: Document access level. Defaults to organization if user belongs to an org, otherwise private.
        :param description: Optional document description.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateDocumentRequest(
            access_level=access_level,
            description=description,
            payload=utils.get_pydantic_model(
                payload, models.CreateDocumentRequestPayload
            ),
            title=title,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1beta/documents",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "multipart", models.CreateDocumentRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CreateDocumentResponseResponse(
                result=unmarshal_json_response(models.CreateDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def get_quota(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetDocumentQuotaResponseResponse:
        r"""Get document quota

        Returns the document quota status for the authenticated user's organization, including the number of documents used, the configured limit, and remaining capacity.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/v1beta/documents/quota",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDocumentQuota",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetDocumentQuotaResponseResponse(
                result=unmarshal_json_response(
                    models.GetDocumentQuotaResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def get_quota_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetDocumentQuotaResponseResponse:
        r"""Get document quota

        Returns the document quota status for the authenticated user's organization, including the number of documents used, the configured limit, and remaining capacity.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/v1beta/documents/quota",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDocumentQuota",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetDocumentQuotaResponseResponse(
                result=unmarshal_json_response(
                    models.GetDocumentQuotaResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def get(
        self,
        *,
        document_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetDocumentResponseResponse:
        r"""Retrieve a document

        Retrieve a document by ID.

        :param document_id: Document ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetDocumentRequest(
            document_id=document_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1beta/documents/{document_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetDocumentResponseResponse(
                result=unmarshal_json_response(models.GetDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        document_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetDocumentResponseResponse:
        r"""Retrieve a document

        Retrieve a document by ID.

        :param document_id: Document ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetDocumentRequest(
            document_id=document_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1beta/documents/{document_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetDocumentResponseResponse(
                result=unmarshal_json_response(models.GetDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def update(
        self,
        *,
        document_id: str,
        description: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UpdateDocumentResponseResponse:
        r"""Update a document

        Update document title and description.

        :param document_id: Document ID to update.
        :param description: New description (max 2000 characters).
        :param title: New title (1-255 characters).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateDocumentRequest(
            document_id=document_id,
            body=models.UpdateDocumentUpdateDocumentRequest(
                description=description,
                title=title,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v1beta/documents/{document_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.UpdateDocumentUpdateDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.UpdateDocumentResponseResponse(
                result=unmarshal_json_response(models.UpdateDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        document_id: str,
        description: OptionalNullable[str] = UNSET,
        title: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UpdateDocumentResponseResponse:
        r"""Update a document

        Update document title and description.

        :param document_id: Document ID to update.
        :param description: New description (max 2000 characters).
        :param title: New title (1-255 characters).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateDocumentRequest(
            document_id=document_id,
            body=models.UpdateDocumentUpdateDocumentRequest(
                description=description,
                title=title,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v1beta/documents/{document_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.UpdateDocumentUpdateDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.UpdateDocumentResponseResponse(
                result=unmarshal_json_response(models.UpdateDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def list_duplicates(
        self,
        *,
        document_id: str,
        file_contents_hash: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListDocumentDuplicatesResponseResponse:
        r"""List duplicate documents

        Finds documents that are visual duplicates of the specified document using perceptual hashing.

        :param document_id: Document ID to check for duplicates.
        :param file_contents_hash: SHA256 hash of the document's visual content for duplicate matching.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListDocumentDuplicatesRequest(
            document_id=document_id,
            file_contents_hash=file_contents_hash,
        )

        req = self._build_request(
            method="GET",
            path="/v1beta/documents/{document_id}/duplicates",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listDocumentDuplicates",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListDocumentDuplicatesResponseResponse(
                result=unmarshal_json_response(
                    models.ListDocumentDuplicatesResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def list_duplicates_async(
        self,
        *,
        document_id: str,
        file_contents_hash: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListDocumentDuplicatesResponseResponse:
        r"""List duplicate documents

        Finds documents that are visual duplicates of the specified document using perceptual hashing.

        :param document_id: Document ID to check for duplicates.
        :param file_contents_hash: SHA256 hash of the document's visual content for duplicate matching.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListDocumentDuplicatesRequest(
            document_id=document_id,
            file_contents_hash=file_contents_hash,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1beta/documents/{document_id}/duplicates",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listDocumentDuplicates",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListDocumentDuplicatesResponseResponse(
                result=unmarshal_json_response(
                    models.ListDocumentDuplicatesResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def export(
        self,
        *,
        document_id: str,
        version_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ExportDocumentResponseResponse:
        r"""Export a document

        Generates a time-limited download URL for the document PDF.

        :param document_id: Document ID.
        :param version_id: Optional version ID. If omitted, exports the current version.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ExportDocumentRequest(
            document_id=document_id,
            version_id=version_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1beta/documents/{document_id}/export",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="exportDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ExportDocumentResponseResponse(
                result=unmarshal_json_response(models.ExportDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def export_async(
        self,
        *,
        document_id: str,
        version_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ExportDocumentResponseResponse:
        r"""Export a document

        Generates a time-limited download URL for the document PDF.

        :param document_id: Document ID.
        :param version_id: Optional version ID. If omitted, exports the current version.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ExportDocumentRequest(
            document_id=document_id,
            version_id=version_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1beta/documents/{document_id}/export",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="exportDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ExportDocumentResponseResponse(
                result=unmarshal_json_response(models.ExportDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def process(
        self,
        *,
        document_id: str,
        async_: Optional[bool] = None,
        version_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ProcessDocumentResponseResponse:
        r"""Process a document

        Triggers the AI processing pipeline (extraction and summarization) for the document's current or specified version.

        :param document_id: Document ID.
        :param async_: If true, returns immediately without waiting for processing to complete.
        :param version_id: Optional version ID. If omitted, processes the current version.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProcessDocumentRequest(
            document_id=document_id,
            body=models.ProcessDocumentProcessDocumentRequest(
                async_=async_,
                version_id=version_id,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1beta/documents/{document_id}/process",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.ProcessDocumentProcessDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="processDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ProcessDocumentResponseResponse(
                result=unmarshal_json_response(
                    models.ProcessDocumentResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def process_async(
        self,
        *,
        document_id: str,
        async_: Optional[bool] = None,
        version_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ProcessDocumentResponseResponse:
        r"""Process a document

        Triggers the AI processing pipeline (extraction and summarization) for the document's current or specified version.

        :param document_id: Document ID.
        :param async_: If true, returns immediately without waiting for processing to complete.
        :param version_id: Optional version ID. If omitted, processes the current version.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProcessDocumentRequest(
            document_id=document_id,
            body=models.ProcessDocumentProcessDocumentRequest(
                async_=async_,
                version_id=version_id,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1beta/documents/{document_id}/process",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.ProcessDocumentProcessDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="processDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ProcessDocumentResponseResponse(
                result=unmarshal_json_response(
                    models.ProcessDocumentResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def transfer_ownership(
        self,
        *,
        document_id: str,
        new_owner_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransferDocumentOwnershipResponseResponse:
        r"""Transfer document ownership

        Transfers ownership of a document to another user in the organization.

        :param document_id: Document ID.
        :param new_owner_id: User ID of the new owner.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TransferDocumentOwnershipRequest(
            document_id=document_id,
            body=models.TransferDocumentOwnershipTransferDocumentOwnershipRequest(
                new_owner_id=new_owner_id,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1beta/documents/{document_id}/transfer",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.TransferDocumentOwnershipTransferDocumentOwnershipRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="transferDocumentOwnership",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TransferDocumentOwnershipResponseResponse(
                result=unmarshal_json_response(
                    models.TransferDocumentOwnershipResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def transfer_ownership_async(
        self,
        *,
        document_id: str,
        new_owner_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransferDocumentOwnershipResponseResponse:
        r"""Transfer document ownership

        Transfers ownership of a document to another user in the organization.

        :param document_id: Document ID.
        :param new_owner_id: User ID of the new owner.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TransferDocumentOwnershipRequest(
            document_id=document_id,
            body=models.TransferDocumentOwnershipTransferDocumentOwnershipRequest(
                new_owner_id=new_owner_id,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1beta/documents/{document_id}/transfer",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.TransferDocumentOwnershipTransferDocumentOwnershipRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="transferDocumentOwnership",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TransferDocumentOwnershipResponseResponse(
                result=unmarshal_json_response(
                    models.TransferDocumentOwnershipResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def trash(
        self,
        *,
        document_id: str,
        body: Union[
            models.TrashDocumentTrashDocumentRequest,
            models.TrashDocumentTrashDocumentRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TrashDocumentResponseResponse:
        r"""Trash a document

        Moves a document to trash and revokes all non-owner access.

        :param document_id: Document ID.
        :param body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TrashDocumentRequest(
            document_id=document_id,
            body=utils.get_pydantic_model(
                body, models.TrashDocumentTrashDocumentRequest
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1beta/documents/{document_id}/trash",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.TrashDocumentTrashDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="trashDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TrashDocumentResponseResponse(
                result=unmarshal_json_response(models.TrashDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def trash_async(
        self,
        *,
        document_id: str,
        body: Union[
            models.TrashDocumentTrashDocumentRequest,
            models.TrashDocumentTrashDocumentRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TrashDocumentResponseResponse:
        r"""Trash a document

        Moves a document to trash and revokes all non-owner access.

        :param document_id: Document ID.
        :param body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TrashDocumentRequest(
            document_id=document_id,
            body=utils.get_pydantic_model(
                body, models.TrashDocumentTrashDocumentRequest
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1beta/documents/{document_id}/trash",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.TrashDocumentTrashDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="trashDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TrashDocumentResponseResponse(
                result=unmarshal_json_response(models.TrashDocumentResponse, http_res),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    def untrash(
        self,
        *,
        document_id: str,
        body: Union[
            models.UntrashDocumentUntrashDocumentRequest,
            models.UntrashDocumentUntrashDocumentRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UntrashDocumentResponseResponse:
        r"""Restore a document from trash

        Restores a document from trash. Previous sharing and access permissions are NOT restored.

        :param document_id: Document ID.
        :param body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UntrashDocumentRequest(
            document_id=document_id,
            body=utils.get_pydantic_model(
                body, models.UntrashDocumentUntrashDocumentRequest
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1beta/documents/{document_id}/untrash",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.UntrashDocumentUntrashDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="untrashDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.UntrashDocumentResponseResponse(
                result=unmarshal_json_response(
                    models.UntrashDocumentResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)

    async def untrash_async(
        self,
        *,
        document_id: str,
        body: Union[
            models.UntrashDocumentUntrashDocumentRequest,
            models.UntrashDocumentUntrashDocumentRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.UntrashDocumentResponseResponse:
        r"""Restore a document from trash

        Restores a document from trash. Previous sharing and access permissions are NOT restored.

        :param document_id: Document ID.
        :param body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UntrashDocumentRequest(
            document_id=document_id,
            body=utils.get_pydantic_model(
                body, models.UntrashDocumentUntrashDocumentRequest
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1beta/documents/{document_id}/untrash",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.UntrashDocumentUntrashDocumentRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="untrashDocument",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.UntrashDocumentResponseResponse(
                result=unmarshal_json_response(
                    models.UntrashDocumentResponse, http_res
                ),
                headers={},
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorResponseData, http_res)
            raise errors.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FactifyDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FactifyDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accessgrantresult import AccessGrantResult, AccessGrantResultTypedDict
from factify.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class GrantDocumentAccessResponseTypedDict(TypedDict):
    r"""GrantDocumentAccessResponse contains the results of granting access."""

    document_id: NotRequired[str]
    r"""Document ID."""
    failed: NotRequired[int]
    r"""Number of failed grants."""
    results: NotRequired[List[AccessGrantResultTypedDict]]
    r"""Results for each recipient."""
    successful: NotRequired[int]
    r"""Number of successful grants."""
    total: NotRequired[int]
    r"""Total number of recipients processed."""


class GrantDocumentAccessResponse(BaseModel):
    r"""GrantDocumentAccessResponse contains the results of granting access."""

    document_id: Optional[str] = None
    r"""Document ID."""

    failed: Optional[int] = None
    r"""Number of failed grants."""

    results: Optional[List[AccessGrantResult]] = None
    r"""Results for each recipient."""

    successful: Optional[int] = None
    r"""Number of successful grants."""

    total: Optional[int] = None
    r"""Total number of recipients processed."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["document_id", "failed", "results", "successful", "total"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organization import Organization, OrganizationTypedDict
from factify.types import BaseModel
from typing_extensions import TypedDict


class AcceptOrganizationInviteResponseTypedDict(TypedDict):
    r"""AcceptOrganizationInviteResponse contains the organization the user joined."""

    organization: OrganizationTypedDict
    r"""Organization represents a Factify organization."""


class AcceptOrganizationInviteResponse(BaseModel):
    r"""AcceptOrganizationInviteResponse contains the organization the user joined."""

    organization: Organization
    r"""Organization represents a Factify organization."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addorganizationmemberresponse import (
    AddOrganizationMemberResponse,
    AddOrganizationMemberResponseTypedDict,
)
from .organizationrole import OrganizationRole
from factify.types import BaseModel
from factify.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Dict, List
from typing_extensions import Annotated, TypedDict


class AddOrganizationMemberAddOrganizationMemberRequestTypedDict(TypedDict):
    r"""AddOrganizationMemberRequest identifies a user to add as a member."""

    role: OrganizationRole
    user_id: str
    r"""User account ID of the member to add.
    Pattern: user_[0-9a-hjkmnp-tv-z]{26}
    """


class AddOrganizationMemberAddOrganizationMemberRequest(BaseModel):
    r"""AddOrganizationMemberRequest identifies a user to add as a member."""

    role: OrganizationRole

    user_id: str
    r"""User account ID of the member to add.
    Pattern: user_[0-9a-hjkmnp-tv-z]{26}
    """


class AddOrganizationMemberRequestTypedDict(TypedDict):
    organization_id: str
    r"""Organization to add the member to.
    Pattern: org_[0-9a-hjkmnp-tv-z]{26}
    """
    body: AddOrganizationMemberAddOrganizationMemberRequestTypedDict


class AddOrganizationMemberRequest(BaseModel):
    organization_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Organization to add the member to.
    Pattern: org_[0-9a-hjkmnp-tv-z]{26}
    """

    body: Annotated[
        AddOrganizationMemberAddOrganizationMemberRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class AddOrganizationMemberResponseResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: AddOrganizationMemberResponseTypedDict


class AddOrganizationMemberResponseResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: AddOrganizationMemberResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from factify.types import BaseModel
from typing_extensions import TypedDict


class VersionRefTypedDict(TypedDict):
    r"""VersionRef is a lightweight reference to a version."""

    id: str
    r"""Version ID.
    Pattern: ver_[0-9a-hjkmnp-tv-z]{26}
    """


class VersionRef(BaseModel):
    r"""VersionRef is a lightweight reference to a version."""

    id: str
    r"""Version ID.
    Pattern: ver_[0-9a-hjkmnp-tv-z]{26}
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .document import Document, DocumentTypedDict
from factify.types import BaseModel
from typing_extensions import TypedDict


class CreateDocumentResponseTypedDict(TypedDict):
    r"""CreateDocumentResponse contains the newly created document."""

    document: DocumentTypedDict
    r"""Document represents a Factify document."""


class CreateDocumentResponse(BaseModel):
    r"""CreateDocumentResponse contains the newly created document."""

    document: Document
    r"""Document represents a Factify document."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .timelineeventtype import TimelineEventType
from .value import Value, ValueTypedDict
from datetime import datetime
from factify.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class TimelineEventTypedDict(TypedDict):
    event_id: NotRequired[str]
    event_type: NotRequired[TimelineEventType]
    occurred_at: NotRequired[datetime]
    payload: NotRequired[Dict[str, Nullable[ValueTypedDict]]]
    r"""`Struct` represents a structured data value, consisting of fields
    which map to dynamically typed values. In some languages, `Struct`
    might be supported by a native representation. For example, in
    scripting languages like JS a struct is represented as an
    object. The details of that representation are described together
    with the proto support for the language.

    The JSON representation for `Struct` is JSON object.
    """


class TimelineEvent(BaseModel):
    event_id: Optional[str] = None

    event_type: Optional[TimelineEventType] = None

    occurred_at: Optional[datetime] = None

    payload: Optional[Dict[str, Nullable[Value]]] = None
    r"""`Struct` represents a structured data value, consisting of fields
    which map to dynamically typed values. In some languages, `Struct`
    might be supported by a native representation. For example, in
    scripting languages like JS a struct is represented as an
    object. The details of that representation are described together
    with the proto support for the language.

    The JSON representation for `Struct` is JSON object.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["event_id", "event_type", "occurred_at", "payload"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


GeneralAccessLevel = Literal[
    "restricted",
    "organization",
    "anyone_with_link",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .generalaccesslevel import GeneralAccessLevel
from .setgeneralaccessresponse import (
    SetGeneralAccessResponse,
    SetGeneralAccessResponseTypedDict,
)
from factify.types import BaseModel
from factify.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Dict, List
from typing_extensions import Annotated, TypedDict


class SetGeneralAccessSetGeneralAccessRequestTypedDict(TypedDict):
    r"""SetGeneralAccessRequest sets the general access level."""

    general_access_level: GeneralAccessLevel


class SetGeneralAccessSetGeneralAccessRequest(BaseModel):
    r"""SetGeneralAccessRequest sets the general access level."""

    general_access_level: GeneralAccessLevel


class SetGeneralAccessRequestTypedDict(TypedDict):
    document_id: str
    r"""Document ID."""
    body: SetGeneralAccessSetGeneralAccessRequestTypedDict


class SetGeneralAccessRequest(BaseModel):
    document_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Document ID."""

    body: Annotated[
        SetGeneralAccessSetGeneralAccessRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class SetGeneralAccessResponseResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: SetGeneralAccessResponseTypedDict


class SetGeneralAccessResponseResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: SetGeneralAccessResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .version import Version, VersionTypedDict
from factify.types import BaseModel
from factify.utils import FieldMetadata, PathParamMetadata
from typing import Dict, List
from typing_extensions import Annotated, TypedDict


class GetVersionRequestTypedDict(TypedDict):
    version_id: str
    r"""Version ID to retrieve.
    Pattern: ver_[0-9a-hjkmnp-tv-z]{26}
    """


class GetVersionRequest(BaseModel):
    version_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Version ID to retrieve.
    Pattern: ver_[0-9a-hjkmnp-tv-z]{26}
    """


class GetVersionResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: VersionTypedDict


class GetVersionResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: Version



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationmember import OrganizationMember, OrganizationMemberTypedDict
from factify.types import BaseModel
from typing_extensions import TypedDict


class UpdateOrganizationMemberResponseTypedDict(TypedDict):
    r"""UpdateOrganizationMemberResponse contains the updated member."""

    member: OrganizationMemberTypedDict
    r"""OrganizationMember represents a member of an organization."""


class UpdateOrganizationMemberResponse(BaseModel):
    r"""UpdateOrganizationMemberResponse contains the updated member."""

    member: OrganizationMember
    r"""OrganizationMember represents a member of an organization."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getuserpreferencesresponse import (
    GetUserPreferencesResponse,
    GetUserPreferencesResponseTypedDict,
)
from factify.types import BaseModel
from typing import Dict, List
from typing_extensions import TypedDict


class GetUserPreferencesResponseResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetUserPreferencesResponseTypedDict


class GetUserPreferencesResponseResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetUserPreferencesResponse
